@@ -51,23 +51,21 @@ opts = [
     cfg.StrOpt('dd_block_size',
                default='1M',
                help='Block size to use when writing to the nodes disk.'),
-    # TODO(dtantsur): remove in Ussuri, then clean up ironic from it.
-    cfg.IntOpt('iscsi_verify_attempts',
-               default=3,
-               help='Maximum attempts to verify an iSCSI connection is '
-                    'active, sleeping 1 second between attempts.',
-               deprecated_for_removal=True,
-               deprecated_reason='Split into [iscsi]verify_attempts and '
-                                 'partition_detection_attempts.'),
     cfg.IntOpt('partition_detection_attempts',
+               default=3,
                min=1,
-               help='Maximum attempts to detect a newly created partition. '
-                    'Defaults to iscsi_verify_attempts, will default to 3 '
-                    'when it is removed.'),
+               help='Maximum attempts to detect a newly created partition.'),
     cfg.IntOpt('partprobe_attempts',
                default=10,
                help='Maximum number of attempts to try to read the '
                     'partition.'),
+    cfg.IntOpt('image_convert_memory_limit',
+               default=1024,
+               help='Memory limit for "qemu-img convert" in MiB. Implemented '
+                    'via the address space resource limit.'),
+    cfg.IntOpt('image_convert_attempts',
+               default=3,
+               help='Number of attempts to convert an image.'),
 ]
 
 CONF = cfg.CONF
@@ -76,16 +74,27 @@ CONF.register_opts(opts, group='disk_utils')
 LOG = logging.getLogger(__name__)
 
 _PARTED_PRINT_RE = re.compile(r"^(\d+):([\d\.]+)MiB:"
-                              "([\d\.]+)MiB:([\d\.]+)MiB:(\w*):(.*):(.*);")
+                              r"([\d\.]+)MiB:([\d\.]+)MiB:(\w*):(.*):(.*);")
 
 CONFIGDRIVE_LABEL = "config-2"
 MAX_CONFIG_DRIVE_SIZE_MB = 64
 
+GPT_SIZE_SECTORS = 33
+
 # Maximum disk size supported by MBR is 2TB (2 * 1024 * 1024 MB)
 MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR = 2097152
 
 # Limit the memory address space to 1 GiB when running qemu-img
-QEMU_IMG_LIMITS = processutils.ProcessLimits(address_space=1 * units.Gi)
+QEMU_IMG_LIMITS = None
+
+
+def _qemu_img_limits():
+    global QEMU_IMG_LIMITS
+    if QEMU_IMG_LIMITS is None:
+        QEMU_IMG_LIMITS = processutils.ProcessLimits(
+            address_space=CONF.disk_utils.image_convert_memory_limit
+            * units.Mi)
+    return QEMU_IMG_LIMITS
 
 
 def list_partitions(device):
@@ -291,8 +300,8 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     # a PrEP partition at the start of the disk. This is an 8 MiB partition
     # with the boot and prep flags set. The bootloader should be installed
     # here.
-    if (cpu_arch.startswith("ppc64") and boot_mode == "bios" and
-        boot_option == "local"):
+    if (cpu_arch.startswith("ppc64") and boot_mode == "bios"
+            and boot_option == "local"):
         LOG.debug("Add PReP boot partition (8 MB) to device: "
                   "%(dev)s for node %(node)s",
                   {'dev': dev, 'node': node_uuid})
@@ -344,9 +353,7 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
 
 def is_block_device(dev):
     """Check whether a device is block or not."""
-    attempts = (CONF.disk_utils.iscsi_verify_attempts
-                if CONF.disk_utils.partition_detection_attempts is None
-                else CONF.disk_utils.partition_detection_attempts)
+    attempts = CONF.disk_utils.partition_detection_attempts
     for attempt in range(attempts):
         try:
             s = os.stat(dev)
@@ -382,14 +389,30 @@ def qemu_img_info(path):
 
     out, err = utils.execute('env', 'LC_ALL=C', 'LANG=C',
                              'qemu-img', 'info', path,
-                             prlimit=QEMU_IMG_LIMITS)
+                             prlimit=_qemu_img_limits())
     return imageutils.QemuImgInfo(out)
 
 
 def convert_image(source, dest, out_format, run_as_root=False):
     """Convert image to other format."""
-    cmd = ('qemu-img', 'convert', '-O', out_format, source, dest)
-    utils.execute(*cmd, run_as_root=run_as_root, prlimit=QEMU_IMG_LIMITS)
+    # TODO(dtantsur): use the retrying library
+    for attempt in range(CONF.disk_utils.image_convert_attempts):
+        cmd = ('qemu-img', 'convert', '-O', out_format, source, dest)
+        try:
+            utils.execute(*cmd, run_as_root=run_as_root,
+                          prlimit=_qemu_img_limits(),
+                          use_standard_locale=True)
+        except processutils.ProcessExecutionError as e:
+            if (('Resource temporarily unavailable' in e.stderr
+                    or 'Cannot allocate memory' in e.stderr)
+                    and attempt < CONF.disk_utils.image_convert_attempts - 1):
+                LOG.debug('Failed to convert image, retrying. Error: %s', e)
+                # Sync disk caches before the next attempt
+                utils.execute('sync')
+            else:
+                raise
+        else:
+            return
 
 
 def populate_image(src, dst, conv_flags=None):
@@ -469,13 +492,24 @@ def destroy_disk_metadata(dev, node_uuid):
     # erasing partition data.
     # This is the same bug as
     # https://bugs.launchpad.net/ironic-python-agent/+bug/1737556
+
+    # Overwrite the Primary GPT, catch very small partitions (like EBRs)
     dd_device = 'of=%s' % dev
-    gpt_backup = get_dev_block_size(dev) - 33
-    dd_seek = 'seek=%i' % gpt_backup
-    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
+    dd_count = 'count=%s' % GPT_SIZE_SECTORS
+    dev_size = get_dev_block_size(dev)
+    if dev_size < GPT_SIZE_SECTORS:
+        dd_count = 'count=%s' % dev_size
+    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, dd_count,
                   run_as_root=True, use_standard_locale=True)
-    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
-                  dd_seek, run_as_root=True, use_standard_locale=True)
+
+    # Overwrite the Secondary GPT, do this only if there could be one
+    if dev_size > GPT_SIZE_SECTORS:
+        gpt_backup = dev_size - GPT_SIZE_SECTORS
+        dd_seek = 'seek=%i' % gpt_backup
+        dd_count = 'count=%s' % GPT_SIZE_SECTORS
+        utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, dd_count,
+                      dd_seek, run_as_root=True, use_standard_locale=True)
+
     # Go ahead and let sgdisk run as well.
     utils.execute('sgdisk', '-Z', dev, run_as_root=True,
                   use_standard_locale=True)
@@ -917,11 +951,11 @@ def create_config_drive_partition(node_uuid, device, configdrive):
         confdrive_mb, confdrive_file = _get_configdrive(configdrive,
                                                         node_uuid)
         if confdrive_mb > MAX_CONFIG_DRIVE_SIZE_MB:
-                raise exception.InstanceDeployFailure(
-                    _('Config drive size exceeds maximum limit of 64MiB. '
-                      'Size of the given config drive is %(size)d MiB for '
-                      'node %(node)s.')
-                    % {'size': confdrive_mb, 'node': node_uuid})
+            raise exception.InstanceDeployFailure(
+                _('Config drive size exceeds maximum limit of 64MiB. '
+                  'Size of the given config drive is %(size)d MiB for '
+                  'node %(node)s.')
+                % {'size': confdrive_mb, 'node': node_uuid})
 
         LOG.debug("Adding config drive partition %(size)d MiB to "
                   "device: %(dev)s for node %(node)s",
@@ -970,8 +1004,8 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                                 "node %(node)s. Creating config drive "
                                 "at the end of the disk %(disk)s.",
                                 {'node': node_uuid, 'disk': device})
-                    startlimit = (MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR -
-                                  MAX_CONFIG_DRIVE_SIZE_MB - 1)
+                    startlimit = (MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR
+                                  - MAX_CONFIG_DRIVE_SIZE_MB - 1)
                     endlimit = MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR - 1
 
                 utils.execute('parted', '-a', 'optimal', '-s', '--', device,
