@@ -511,6 +511,12 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         return ['parted', '-a', 'optimal', '-s', dev,
                 '--', 'unit', 'MiB', 'mklabel', label]
 
+    def _add_efi_sz(self, x):
+        return str(x + self.efi_size)
+
+    def _add_bios_sz(self, x):
+        return str(x + self.bios_size)
+
     def _test_make_partitions(self, mock_exc, boot_option, boot_mode='bios',
                               disk_label=None, cpu_arch=""):
         mock_exc.return_value = ('', '')
@@ -521,14 +527,13 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                                    cpu_arch=cpu_arch)
 
         if boot_option == "local" and boot_mode == "uefi":
-            add_efi_sz = lambda x: str(x + self.efi_size)
             expected_mkpart = ['mkpart', 'primary', 'fat32', '1',
-                               add_efi_sz(1),
+                               self._add_efi_sz(1),
                                'set', '1', 'boot', 'on',
                                'mkpart', 'primary', 'linux-swap',
-                               add_efi_sz(1), add_efi_sz(513), 'mkpart',
-                               'primary', '', add_efi_sz(513),
-                               add_efi_sz(1537)]
+                               self._add_efi_sz(1), self._add_efi_sz(513),
+                               'mkpart', 'primary', '', self._add_efi_sz(513),
+                               self._add_efi_sz(1537)]
         else:
             if boot_option == "local":
                 if disk_label == "gpt":
@@ -539,14 +544,15 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                                            '9', '521', 'mkpart', 'primary',
                                            '', '521', '1545']
                     else:
-                        add_bios_sz = lambda x: str(x + self.bios_size)
                         expected_mkpart = ['mkpart', 'primary', '', '1',
-                                           add_bios_sz(1),
+                                           self._add_bios_sz(1),
                                            'set', '1', 'bios_grub', 'on',
                                            'mkpart', 'primary', 'linux-swap',
-                                           add_bios_sz(1), add_bios_sz(513),
+                                           self._add_bios_sz(1),
+                                           self._add_bios_sz(513),
                                            'mkpart', 'primary', '',
-                                           add_bios_sz(513), add_bios_sz(1537)]
+                                           self._add_bios_sz(513),
+                                           self._add_bios_sz(1537)]
                 elif cpu_arch.startswith('ppc64'):
                     expected_mkpart = ['mkpart', 'primary', '', '1', '9',
                                        'set', '1', 'boot', 'on',
@@ -563,8 +569,8 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                                    '513', 'mkpart', 'primary', '', '513',
                                    '1537']
         self.dev = 'fake-dev'
-        parted_cmd = (self._get_parted_cmd(self.dev, disk_label) +
-                      expected_mkpart)
+        parted_cmd = (self._get_parted_cmd(self.dev, disk_label)
+                      + expected_mkpart)
         parted_call = mock.call(*parted_cmd, use_standard_locale=True,
                                 run_as_root=True, check_exit_code=[0])
         fuser_cmd = ['fuser', 'fake-dev']
@@ -794,6 +800,58 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
         mock_exec.assert_has_calls(expected_call)
 
+    def test_destroy_disk_metadata_ebr(self, mock_exec):
+        expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('blockdev', '--getsz', 'fake-dev',
+                                    check_exit_code=[0],
+                                    run_as_root=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=2',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('sgdisk', '-Z', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True)]
+        mock_exec.side_effect = iter([
+            (None, None),
+            ('2\n', None),  # an EBR is 2 sectors
+            (None, None),
+            (None, None),
+            (None, None),
+            (None, None)])
+        disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
+        mock_exec.assert_has_calls(expected_calls)
+
+    def test_destroy_disk_metadata_tiny_partition(self, mock_exec):
+        expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('blockdev', '--getsz', 'fake-dev',
+                                    check_exit_code=[0],
+                                    run_as_root=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=33',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=33', 'seek=9',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('sgdisk', '-Z', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True)]
+        mock_exec.side_effect = iter([
+            (None, None),
+            ('42\n', None),
+            (None, None),
+            (None, None),
+            (None, None),
+            (None, None)])
+        disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
+        mock_exec.assert_has_calls(expected_calls)
+
 
 @mock.patch.object(utils, 'execute', autospec=True)
 class GetDeviceBlockSizeTestCase(base.IronicLibTestCase):
@@ -1009,15 +1067,6 @@ class OtherFunctionTestCase(base.IronicLibTestCase):
                           disk_utils.is_block_device, device)
         mock_os.assert_has_calls([mock.call(device)] * 2)
 
-    @mock.patch.object(os, 'stat', autospec=True)
-    def test_is_block_device_deprecated_attempts(self, mock_os):
-        CONF.set_override('iscsi_verify_attempts', 4, group='disk_utils')
-        device = '/dev/disk/by-path/ip-1.2.3.4:5678-iscsi-iqn.fake-lun-9'
-        mock_os.side_effect = OSError
-        self.assertRaises(exception.InstanceDeployFailure,
-                          disk_utils.is_block_device, device)
-        mock_os.assert_has_calls([mock.call(device)] * 4)
-
     @mock.patch.object(imageutils, 'QemuImgInfo', autospec=True)
     @mock.patch.object(os.path, 'exists', return_value=False, autospec=True)
     def test_qemu_img_info_path_doesnt_exist(self, path_exists_mock,
